package com.as.lyons.wcf.svc.atm;

import java.net.URL;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.CommandLineRunner;
import org.springframework.boot.SpringApplication;
import org.springframework.boot.autoconfigure.SpringBootApplication;
import org.springframework.context.annotation.Bean;
import org.springframework.util.Assert;

import com.as.lyons.wcf.svc.atm.client.ATMServiceWCF;
import com.as.lyons.wcf.svc.atm.client.IATMService;

/**
 * 
 *
 */
@SpringBootApplication
public class App implements LogonProperties {
	private static final Logger log = LoggerFactory.getLogger(App.class);

	private static final Map<String, AtmServiceCall> svcMethods = new HashMap<>();

	private static String appStartCommand = "java -cp <...> <your class> ";

	public static void main(String[] args) {
		setAppStartCommand();
		svcMethods.put("help", new HelpAtmServiceCall());
		svcMethods.put("VerifyAccount", new VerifyAccountServiceCall());
		svcMethods.put("VerifyAccountWithDetails", new VerifyAccountWithDetailsServiceCall());
		svcMethods.put("IsParticipant", new IsParticipantServiceCall());
		svcMethods.put("DescribeStatusCode", new DescribeStatusCodeServiceCall());
		SpringApplication.run(App.class, args);
	}

	@Value("${company.id}")
	private int companyId;
	@Value("${company.user.name}")
	private String userName;
	@Value("${company.user.password}")
	private String password;

	@Bean
	public CommandLineRunner run(IATMService svc) throws Exception {
		final LogonProperties logonProps = this;
		return args -> {
			log.info("Begin method execution");
			System.out.println("\n\n\n");
			String[] augArgs = new String[0];
			String methodName = null;
			if (args != null) {
				List<String> argLst = new ArrayList<>(args.length);
				for (String s : args) {
					if (s != null && !s.startsWith("--")) {
						if (methodName == null) {
							methodName = s;
						} else {
							argLst.add(s);
						}
					}
				}
				augArgs = argLst.toArray(augArgs);
			}
			if (augArgs.length == 0) {
				printHelp("Invalid number of arguments provided!");
				return;
			}
			AtmServiceCall method = svcMethods.get(methodName);
			if (method == null) {
				printHelp("The provided method is not recognized!");
				return;
			}
			try {
				method.call(svc, logonProps, methodName, augArgs);
			} catch (StopProcesingException e) {
			}
			System.out.println("\n\n\n");
			log.info("End method execution");
		};
	}

	protected void printHelp(String error) {
		if (error != null) {
			System.out.println(error);
		}
		System.out.println(new StringBuilder(500).append("Use:\n\t").append(appStartCommand)
				.append("help <method name>").toString());
		System.out.println(new StringBuilder(500).append("\nor:\n\t").append(appStartCommand)
				.append("<method name> <arguments>").toString());
		System.out.println("\twhere <method name> can be:");
		for (String method : svcMethods.keySet()) {
			if (!"help".equals(method)) {
				System.out.println("\t\t".concat(method));
			}
		}
		System.out.println("\n\n\n");
	}

	@Override
	public int getCompanyId() {
		return companyId;
	}

	@Override
	public String getUserName() {
		return userName;
	}

	@Override
	public String getPassword() {
		return password;
	}

	@Override
	public String getAppStartCommand() {
		return appStartCommand;
	}

	@Override
	public Map<String, AtmServiceCall> getSvcMethods() {
		return Collections.unmodifiableMap(svcMethods);
	}

	private static ATMServiceWCF createServiceWCF(String singleWsdlUrl) throws StopProcesingException {
		ATMServiceWCF wcf = null;
		try {
			wcf = new ATMServiceWCF(new URL(singleWsdlUrl));
		} catch (Exception e) {
			log.error("\nError creating the ATMServiceWCF: {}", e.getMessage());
			log.error("Exception:", e);
			throw new StopProcesingException();
		}
		return wcf;
	}

	@Bean
	private static IATMService prepareService(@Value("${service.single.wsdl.url}") String singleWsdlUrl)
			throws StopProcesingException {
		ATMServiceWCF wcf = createServiceWCF(singleWsdlUrl);
		IATMService svc = null;
		try {
			svc = wcf.getBasicHttpBindingIATMService();
			Assert.notNull(svc, "The returned IATMService is null");
		} catch (RuntimeException e) {
			System.out.println("\nError obtaining the basic HTTP binding to IATMService: ".concat(e.getMessage()));
			e.printStackTrace();
			throw new StopProcesingException();
		}
		return svc;
	}

	private static void setAppStartCommand() {
		ClassLoader loader = App.class.getClassLoader();
		if (loader == null) {
			// Try the bootstrap classloader - obtained from the ultimate parent
			// of the System Class Loader.
			loader = ClassLoader.getSystemClassLoader();
			while (loader != null && loader.getParent() != null) {
				loader = loader.getParent();
			}
		}
		if (loader == null) {
			return;
		}
		String name = App.class.getCanonicalName();
		URL resource = loader.getResource(name.replace(".", "/") + ".class");
		if (resource == null) {
			return;
		}

		String resStr = resource.toString();
		String jar = null;
		if (resStr.startsWith("jar:")) {
			int jarIdx = resStr.indexOf(".jar!");
			if (jarIdx > 0) {
				resStr = resStr.substring(0, jarIdx + 4);
				jarIdx = resStr.lastIndexOf('/');
				if (jarIdx >= 0) {
					jar = resStr.substring(jarIdx + 1);
				} else {
					jar = resStr;
				}
			}
		}
		StringBuilder sb = new StringBuilder(500).append("java ");
		if (jar == null) {
			sb.append("-cp <...> ").append(name);
		} else {
			sb.append("-jar ").append(jar);
		}
		appStartCommand = sb.append(" ").toString();
	}
}
